local NekoLoot = LibStub("AceAddon-3.0"):GetAddon("NekoLoot")
local L = LibStub("AceLocale-3.0"):GetLocale("NekoLoot")
local LBF = LibStub("LibButtonFacade", true)

------
-- constants
local GRPL = "Interface\\Buttons\\UI-GroupLoot-"
local MINI_B = "Interface\\Buttons\\UI-Panel-MinimizeButton-"

local dialogBackdrop = {
	bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background", tile = true, tileSize = 32,
	edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border", edgeSize = 32,
	insets = {left = 11, right = 12, top = 12, bottom = 11},
}


local backdrop = {
	bgFile = "Interface\\ChatFrame\\ChatFrameBackground", tile = true, tileSize = 16,
	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16,
	insets = {left = 4, right = 4, top = 4, bottom = 4},
}


local gradientOn = {"VERTICAL", .1, .1, .1, 0, .25, .25, .25, 1}


local function CreateDialogFrame(dialogName)
	local dialogFrame = CreateFrame("Frame", dialogName, UIParent)
	dialogFrame:SetFrameStrata("DIALOG")
	dialogFrame:SetToplevel(true)
	dialogFrame:EnableMouse(true)
	dialogFrame:SetClampedToScreen(true)
	dialogFrame:SetWidth(360)
	dialogFrame:SetHeight(160)
	dialogFrame:SetBackdrop(dialogBackdrop)
	dialogFrame:SetPoint("TOP", 0, -100)
	dialogFrame:Hide()
	return dialogFrame
end


-- creates an error dialog
-- dialogName		: name of the created dialog
-- dialogMsg		: message to display on the dialog
-- buttonText		: text to display on the button
-- buttonPos		: position of the clickable button
-- buttonHandler	: function to call when button is clicked
function NekoLoot:CreateDialog(dialogName, dialogMsg, buttonText, buttonPos, buttonHandler)
	if not self.dialog or (self.dialog:GetName() ~= dialogName) then
		local dialogFrame = CreateDialogFrame(dialogName)

		titleRegion = dialogFrame:CreateTitleRegion()
		titleRegion:SetAllPoints(dialogFrame)

		header = dialogFrame:CreateTexture(nil, "ARTWORK")
		header:SetTexture("Interface\\DialogFrame\\UI-DialogBox-Header")
		header:SetWidth(256); header:SetHeight(64)
		header:SetPoint("TOP", 0, 12)
		
		title = dialogFrame:CreateFontString("ARTWORK")
		title:SetFontObject("GameFontNormal")
		title:SetPoint("TOP", header, "TOP", 0, -14)
		title:SetText("NekoLoot")

		desc = dialogFrame:CreateFontString("ARTWORK")
		desc:SetFontObject("GameFontHighlight")
		desc:SetJustifyV("TOP")
		desc:SetJustifyH("LEFT")
		desc:SetPoint("TOPLEFT", 18, -32)
		desc:SetPoint("BOTTOMRIGHT", -18, 48)
		desc:SetText(dialogMsg)
	
		button = CreateFrame("CheckButton", "NekoLootErrorDialog", dialogFrame, "OptionsButtonTemplate")
		_G[button:GetName() .. "Text"]:SetText(buttonText)
		
		button:SetScript("OnClick", function(self) 
			buttonHandler()
			end)
			
		if buttonPos == "BOTTOM" then button:SetPoint(buttonPos, 0, 14)
		else button:SetPoint(buttonPos, -14, 14) end
		
		self.dialog = dialogFrame
	end
	self.dialog:Show()
end


-- creates a GameTooltip to scan item tooltips
function NekoLoot:CreateTooltip()
	self.tooltip = CreateFrame("GameTooltip", "NekoLootTT", UIParent, "GameTooltipTemplate")
end


------
-- Sets a set of scripts to a button. Which script are set depends on the type
-- of the button. Three types are supportet: lootToken, rollToken and rollButton.
--
-- button			: button to add scripts to
-- bType			: type of the button
local function SetScripts(butt, bType)
	NLRoll = NekoLoot:GetModule("NekoLootRoll", true)

	if bType == "lootFrame" then
		butt:SetScript("OnShow", function(self) NekoLoot:OnShow() end)
		butt:SetScript("OnHide", function(self) NekoLoot:OnHide() end)
		butt:SetScript("OnDragStart", StartMoving)
		butt:SetScript("OnDragStop", StopMoving)
	elseif bType == "lootToken" then
		butt:SetScript("OnDragStart", StartMoving)
		butt:SetScript("OnDragStop", StopMoving)
		butt:SetScript("OnEnter", function(self) NekoLoot:OnEnter(self) end)
		butt:SetScript("OnUpdate", function(self, elapsed) CursorOnUpdate(self) end)
	
	elseif bType == "rollToken" and NLRoll then
		butt:SetScript("OnEnter", function(self) NLRoll:OnEnter(self) end)
		butt:SetScript("OnLeave", function(self) NLRoll:OnLeave(self) end)
		butt:SetScript("OnUpdate", function(self, elapsed) CursorOnUpdate(self) end)
	
	elseif bType == "rollButton" and NLRoll then
		butt:SetScript("OnEnter", function(self) NLRoll:ShowTooltip(self) end)
		butt:SetScript("OnLeave", function(self) GameTooltip:Hide() end)
		butt:SetScript("OnClick", function(self) NLRoll:OnClickRoll(self) end)
	end
end


------
-- 
function NekoLoot:ApplySkin(frame, bgColor, borderColor)
	if not frame.tfade then 
		frame.tfade = frame:CreateTexture(nil, "BORDER")
	end
	
	frame:SetBackdrop(backdrop)
	frame:SetBackdropColor(unpack(bgColor))
	frame:SetBackdropBorderColor(unpack(borderColor))

	frame.tfade:SetTexture("Interface\\ChatFrame\\ChatFrameBackground")
	frame.tfade:SetPoint("TOPLEFT", frame, "TOPLEFT", 4, -4)
	frame.tfade:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", -4, 4) 

	frame.tfade:SetBlendMode("ADD")
	frame.tfade:SetGradientAlpha(unpack(gradientOn)) 
end


------
-- 
local function AddCloseButton(parent)
	local closeButt = CreateFrame("Button", "NLCloseButton", parent)

	closeButt:SetScript("OnClick", function(self) NekoLoot:LootClosed() end)
	closeButt:SetFrameLevel(8)
	closeButt:SetWidth(32)
	closeButt:SetHeight(32)
	closeButt:SetNormalTexture(MINI_B .. "Up")
	closeButt:SetPushedTexture(MINI_B .. "Down")
	closeButt:SetHighlightTexture(MINI_B .. "-Highlight")
	closeButt:ClearAllPoints()
	closeButt:SetPoint("TOPRIGHT", NekoLootFrame, "TOPRIGHT", 3, 3)
	closeButt:SetHitRectInsets(5, 5, 5, 5)
	closeButt:Show()
end


------
-- Creates the loot frame with ten empty loot token, that all are hidden
function NekoLoot:CreateLootFrame()
	local db = self.db.profile

	-- Frame Eigenschaften
	local lootFrame = CreateFrame("Frame", "NekoLootFrame", UIParent)
	
	-- Hintergrund
	-- self:ApplySkin(lootFrame, db.frame.backgroundColor, db.frame.borderColor)
	
	-- Events und Handler
	lootFrame:RegisterForDrag("LeftButton")
	SetScripts(lootFrame, "lootFrame")

	AddCloseButton(lootFrame)

	-- Ausrichtung
   	lootFrame:ClearAllPoints()
	lootFrame:SetPoint(
		db.frame.anchorPoint,
		db.frame.parent,
		db.frame.parentAnchor,
		db.frame.xOffSet,
		db.frame.yOffSet
	)

	lootFrame:Hide()
	return lootFrame
end


------
-- Add to lootframe a single empty loot token
--
-- parent			: lootFrame where the token should be added
-- index			: index of the loot token
function NekoLoot:CreateLootToken(lootFrame, index)
	local frame = CreateFrame("Frame", "NekoLootButtonFrame"..index, lootFrame)
	local buttonName = "NekoLootButton"..index
	local button = CreateFrame("Button", buttonName, frame, "LootButtonTemplate")
	button:DisableDrawLayer("ARTWORK")

	local desc = button:CreateFontString(buttonName.."Description")
	desc:SetDrawLayer("OVERLAY")
	desc:SetJustifyH("LEFT")
	button.desc = desc

	local text = _G[buttonName.."Text"]
	text:SetDrawLayer("OVERLAY")
	button.text = text

	local bind = button:CreateFontString(buttonName.."Bind")
	bind:SetDrawLayer("OVERLAY")
	bind:SetJustifyH("CENTER")
	bind:SetPoint("BOTTOMLEFT",  button, "BOTTOMLEFT", 3, 3)
	button.bind = bind

	-- set borders
	button.border = self:QualityBorder(button)
	local fBorder = self:QualityBorder(frame)
	fBorder:SetHeight(fBorder:GetHeight() -3)
	fBorder:SetPoint("CENTER", frame, "CENTER", 4, .5)
	fBorder:SetAlpha(0.3)
	frame.border = fBorder

	frame:SetHeight(button:GetHeight() + 1)
	
	button:ClearAllPoints()
	button:SetPoint("LEFT", frame, "LEFT")
	button:RegisterForDrag("LeftButton")
	SetScripts(button, "lootToken")
	frame.button = button

	-- update height of the parent frame
	lootFrame:SetHeight(lootFrame:GetHeight() + frame:GetHeight())

	frame:Hide()
	button:Hide()
	return frame
end


function NekoLoot:AddLBFGroup(button, groupName, optTab)
	if LBF then
		button.LBFGroup = LBF:Group("NekoLoot", groupName)
		button.LBFGroup.SkinID = optTab.skinID
		button.LBFGroup.Backdrop = optTab.backdrop
		button.LBFGroup.Gloss = optTab.gloss
		button.LBFGroup.Colors = optTab.colors
	
		LBF:RegisterSkinCallback("NekoLoot", self.SkinChanged, self)
	end
end


------
-- Creates the anchor of all roll frames.
--
-- options			: set of options from db
function NekoLoot:CreatRollAnchor(options)
	local rollAnc = CreateFrame("Button", "NekoLootRollAnchor", UIParent)
	rollAnc:SetFrameStrata(options.strata)
	rollAnc:EnableMouse(not options.lock)
	rollAnc:SetMovable(not options.lock)
	rollAnc:SetScale(options.scale)
	rollAnc:SetAlpha(options.alpha)
	rollAnc:SetWidth(options.width)
	rollAnc:SetHeight(options.height)

	self:ApplySkin(rollAnc, options.ancBGColor, options.ancBorderColor)

	local text = rollAnc:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	text:SetAllPoints(rollAnc)
	text:SetText("NekoLootRoll - " .. L["Anchor"])

	rollAnc:RegisterForDrag("LeftButton")
	rollAnc:SetScript("OnDragStart", function(self) NekoLootRoll:StartMoving() end)
	rollAnc:SetScript("OnDragStop", function(self) NekoLootRoll:StopMoving() end)

	-- Ausrichtung
   	rollAnc:ClearAllPoints()
	rollAnc:SetPoint(
		options.anchorPoint,
		options.parent,
		options.parentAnchor,
		options.xOffSet,
		options.yOffSet
	)

	rollAnc:Hide()
	return rollAnc
end


local function CreateRollButton(parent, texture, highlight, rollType, tipText, ...)
	local button = CreateFrame("Button", nil, parent)
	button:SetPoint(...)
	button:SetWidth(28)
	button:SetHeight(28)
	
	button:SetNormalTexture(GRPL .. texture .."-Up")
	button:SetPushedTexture(GRPL .. texture .."-Down")
	if highlight then
		button:SetHighlightTexture(GRPL .. texture .."-Highlight") end
	
	button.rollType = rollType
	button.parent = parent
	button.tipText = tipText
	
	SetScripts(button, "rollButton")

	button:SetMotionScriptsWhileDisabled(true)
	local txt = button:CreateFontString(nil, nil, "GameFontHighlightSmallOutline")
	txt:SetPoint("CENTER", 0, rollType == 2 and 1 or rollType == 0 and -1.2 or 0)
	return button, txt
end


function NekoLoot:CreateRollToken(options)
	NekoLootRoll = NekoLoot:GetModule("NekoLootRoll", true)
	if not NekoLootRoll then return end

	local frame = CreateFrame("Frame", nil, UIParent)
	self:ApplySkin(frame, options.tokenBGColor, options.tokenBorderColor)

	local button = CreateFrame("Button", nil, frame)
	button:SetPoint("LEFT", 5, 0)
	button:SetWidth(24)
	button:SetHeight(24)
	button:SetNormalTexture("Interface\\Buttons\\UI-Quickslot2")
	button:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square")
	button:GetHighlightTexture():SetBlendMode("ADD")
	SetScripts(button, "rollToken")
	frame.button = button

	local buttonborder = CreateFrame("Frame", nil, button)
	buttonborder:SetWidth(32)
	buttonborder:SetHeight(32)
	buttonborder:SetPoint("CENTER", button, "CENTER")
	buttonborder:SetBackdrop(backdrop)
	buttonborder:SetBackdropColor(1, 1, 1, 0)
	frame.buttonborder = buttonborder

	local status = CreateFrame("StatusBar", nil, frame)
	status:SetPoint("TOPRIGHT", frame, "TOPRIGHT", -4, -4)
	status:SetPoint("BOTTOM", frame, "BOTTOM", 0, 4)
	status:SetPoint("LEFT", frame.button, "RIGHT", -1, 0)
	status:SetScript("OnUpdate", function(self) NekoLootRoll:StatusUpdate(self) end)
	status:SetFrameLevel(status:GetFrameLevel()-1)
	status:SetStatusBarTexture("Interface\\AddOns\\NekoLoot\\bg_rollframe.tga")
	status:SetStatusBarColor(.8, .8, .8, .9)
	status.parent = frame
	frame.status = status

	local spark = frame:CreateTexture(nil, "OVERLAY")
	spark:SetWidth(14)
	spark:SetHeight(35)
	spark:SetTexture("Interface\\CastingBar\\UI-CastingBar-Spark")
	spark:SetBlendMode("ADD")
	status.spark = spark

	-- create roll buttons
	local need, needtext = CreateRollButton(frame, "Dice", true, 1, NEED, 
		"LEFT", frame.button, "RIGHT", 5, -1)
	local greed, greedtext = CreateRollButton(frame, "Coin", true, 2, GREED, 
		"LEFT", need, "RIGHT", 0, -1)
	local dis, detext = CreateRollButton(frame, "DE", true, 3, ROLL_DISENCHANT,
		"LEFT", greed, "RIGHT", 0, -1)
	local pass, passtext = CreateRollButton(frame, "Pass", false, 0, PASS,
		"LEFT", dis or greed, "RIGHT", 0, 2.2)

	frame.needButt, frame.greedButt, frame.disButt = need, greed, dis
	frame.need, frame.greed, frame.pass, frame.dis = needtext, greedtext, passtext, detext

	local bind = frame:CreateFontString()
	bind:SetPoint("LEFT", pass, "RIGHT", 3, 1)
	bind:SetFont("Fonts\\FRIZQT__.TTF", 13, "THICKOUTLINE")
	frame.fsbind = bind

	local loot = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	loot:SetPoint("LEFT", bind, "RIGHT", 0, .12)
	loot:SetPoint("RIGHT", frame, "RIGHT", -5, 0)
	loot:SetHeight(16)
	loot:SetJustifyH("LEFT")
	frame.fsloot = loot

	frame.rolls = {}

	frame:Hide()
	return frame
end